﻿package {

	import flash.display.Bitmap;
	import flash.display.Loader;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.filesystem.File;
	import flash.net.URLRequest;
	import flash.system.ApplicationDomain;
	import flash.system.LoaderContext;
	import flash.system.MessageChannel;
	import flash.system.Worker;
	import flash.system.WorkerDomain;
	import flash.utils.ByteArray;
	import flash.utils.setTimeout;
 
 
    [SWF(frameRate = "60", width = "1024", height = "468", backgroundColor = "0xFFFFFF")]
     
    public class Main extends Sprite {
         
        public var worker:Worker;
        public var msgChannelMainToImageWriterWorker:MessageChannel;
		public var msgChannelImageWriterToMainWorker:MessageChannel;
        public var workerByteArrayShared:ByteArray;
		
		[Embed(source="worker0.png")]
		private const Image0:Class;
		
		[Embed(source="worker1.jpg")]
		private const Image1:Class;
		
		[Embed(source="worker2.png")]
		private const Image2:Class;
		
		[Embed(source="worker3.jpg")]
		private const Image3:Class;
        
		private var bitmapsToEncode:Array;
         
        public function Main() {
            super();
             
            var workerLoader:Loader = new Loader();
 
            //we specify a loader context for managing SWF loading on iOS.
            var loaderContext:LoaderContext = new LoaderContext(false, ApplicationDomain.currentDomain);
 
            workerLoader.contentLoaderInfo.addEventListener(Event.COMPLETE, _onImageWriterWorkerLoaded);
            workerLoader.load(new URLRequest("workers/ImageWriterWorker.swf"), loaderContext);
        }
 
        private function _onImageWriterWorkerLoaded(evt:Event):void {
             
            evt.target.removeEventListener(Event.COMPLETE, _onImageWriterWorkerLoaded);
             
            var workerBytes:ByteArray = evt.target.bytes;
            worker = WorkerDomain.current.createWorker(workerBytes, true);
            // we set the latest arguments to true, because we want to be able to save on disk via the Worker.
             
            msgChannelMainToImageWriterWorker = Worker.current.createMessageChannel(worker);
			msgChannelImageWriterToMainWorker = worker.createMessageChannel(Worker.current);
			msgChannelImageWriterToMainWorker.addEventListener(Event.CHANNEL_MESSAGE, messagesFromImageWriterWorker);

			worker.setSharedProperty("mainToImageWriterWorker", msgChannelMainToImageWriterWorker);
			worker.setSharedProperty("imageWriterWorkerToMain", msgChannelImageWriterToMainWorker);
 
            workerByteArrayShared = new ByteArray();
            workerByteArrayShared.shareable = true; // we share the byte array through the worker
            worker.setSharedProperty("imageBytes", workerByteArrayShared);
             
            worker.start();
			
			setTimeout(function():void {
				
				bitmapsToEncode = [];
			
				bitmapsToEncode.push([new Image0(), File.applicationStorageDirectory.resolvePath("images/0.png").nativePath]);
				bitmapsToEncode.push([new Image1(), File.applicationStorageDirectory.resolvePath("images/1.jpg").nativePath]);
				bitmapsToEncode.push([new Image2(), File.applicationStorageDirectory.resolvePath("images/2.png").nativePath]);
				bitmapsToEncode.push([new Image3(), File.applicationStorageDirectory.resolvePath("images/3.jpg").nativePath]);
				
				processBitmapQueueToImageWriterWorker();
			
			}, 200);
		}
		
		public function processBitmapQueueToImageWriterWorker():void {
			
			if (bitmapsToEncode.length < 1) {
				trace("queue finished");
				return;
			}
			
			var tab:Array = bitmapsToEncode.shift();
			
			var bitmap:Bitmap = tab[0];
			
			workerByteArrayShared.clear();
			
			bitmap.bitmapData.copyPixelsToByteArray(bitmap.bitmapData.rect, workerByteArrayShared);
					
			msgChannelMainToImageWriterWorker.send({width:bitmap.width, height:bitmap.height, path:tab[1]});
		}

		public function messagesFromImageWriterWorker(evt:Event):void {
			
			if (msgChannelImageWriterToMainWorker.messageAvailable) {
				
				if (msgChannelImageWriterToMainWorker.receive() == "IMAGE_SAVED") {
					
					processBitmapQueueToImageWriterWorker();
				}
			}
		}
    }
}